@@ -1,5 +1,5 @@
 /* **************************************************************************
- * Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,8 +28,8 @@
 #include <fmt/core.h>
 #include <fmt/ostream.h>
 
-#include "rocblascommon/program_options.hpp"
-#include "rocsolver_dispatcher.hpp"
+#include "common/misc/program_options.hpp"
+#include "common/misc/rocsolver_dispatcher.hpp"
 
 using namespace roc;
 
@@ -617,7 +617,9 @@ try
     // print help message
     if(vm.count("help"))
     {
-        fmt::print("{}{}\n", help_str, desc);
+        std::stringstream desc_ss{};
+        desc_ss << desc;
+        fmt::print("{}{}\n", help_str, desc_ss.str());
         return 0;
     }
 
