@@ -4,7 +4,7 @@
  *     Univ. of Tennessee, Univ. of California Berkeley,
  *     Univ. of Colorado Denver and NAG Ltd..
  *     June 2017
- * Copyright (C) 2019-2022 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2019-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -37,6 +37,8 @@
 #include "rocblas.hpp"
 #include "rocsolver/rocsolver.h"
 
+ROCSOLVER_BEGIN_NAMESPACE
+
 template <bool BATCHED, typename T>
 void rocsolver_latrd_getMemorySize(const rocblas_int n,
                                    const rocblas_int k,
@@ -56,7 +58,8 @@ void rocsolver_latrd_getMemorySize(const rocblas_int n,
         return;
     }
 
-    size_t w_temp;
+    size_t n1 = 0, n2 = 0;
+    size_t w1 = 0, w2 = 0, w3 = 0;
 
     // size of scalars (constants) for rocblas calls
     *size_scalars = sizeof(T) * 3;
@@ -68,11 +71,20 @@ void rocsolver_latrd_getMemorySize(const rocblas_int n,
         *size_workArr = 0;
 
     // extra requirements for calling larfg
-    rocsolver_larfg_getMemorySize<T>(n, batch_count, size_work, size_norms);
+    rocsolver_larfg_getMemorySize<T>(n, batch_count, &w1, &n1);
 
     // extra requirements for calling symv/hemv
-    rocblasCall_symv_hemv_mem<BATCHED, T>(n, batch_count, &w_temp);
-    *size_work = std::max(*size_work, w_temp);
+    rocblasCall_symv_hemv_mem<BATCHED, T>(n, batch_count, &w2);
+
+    // size of re-usable workspace
+    // TODO: replace with rocBLAS call
+    constexpr int ROCBLAS_DOT_NB = 512;
+    w3 = n > 2 ? (n - 2) / ROCBLAS_DOT_NB + 2 : 1;
+    w3 *= sizeof(T) * batch_count;
+    n2 = sizeof(T) * batch_count;
+
+    *size_norms = std::max(n1, n2);
+    *size_work = std::max({w1, w2, w3});
 }
 
 template <typename T, typename S, typename U>
@@ -103,7 +115,7 @@ rocblas_status rocsolver_latrd_argCheck(rocblas_handle handle,
         return rocblas_status_continue;
 
     // 3. invalid pointers
-    if((n && !A) || (n && !E) || (n && !tau) || (n * k && !W))
+    if((n && !A) || (n && !E) || (n && !tau) || (n && k && !W))
         return rocblas_status_invalid_pointer;
 
     return rocblas_status_continue;
@@ -191,7 +203,7 @@ rocblas_status rocsolver_latrd_template(rocblas_handle handle,
 
             // generate Householder reflector to work on column j
             rocsolver_larfg_template(handle, n - j - 1, A, shiftA + idx2D(j + 1, j, lda), A,
-                                     shiftA + idx2D(min(j + 2, n - 1), j, lda), 1, strideA,
+                                     shiftA + idx2D(std::min(j + 2, n - 1), j, lda), 1, strideA,
                                      (tau + j), strideP, batch_count, work, norms);
 
             // copy to E(j) the corresponding off-diagonal element of A, which is set to 1
@@ -339,3 +351,5 @@ rocblas_status rocsolver_latrd_template(rocblas_handle handle,
     rocblas_set_pointer_mode(handle, old_mode);
     return rocblas_status_success;
 }
+
+ROCSOLVER_END_NAMESPACE
