@@ -39,6 +39,8 @@
 #include "roclapack_syev_heev.hpp"
 #include "rocsolver/rocsolver.h"
 
+ROCSOLVER_BEGIN_NAMESPACE
+
 /************** Kernels and device functions for small size*******************/
 /*****************************************************************************/
 
@@ -166,6 +168,7 @@ __device__ void run_syevj(const rocblas_int dimx,
         local_diag += std::real(sines_diag[i]);
     }
     S tolerance = (local_res + local_diag) * abstol * abstol;
+    S small_num = get_safemin<S>() / eps;
 
     // execute sweeps
     rocblas_int count = (half_n - 1) / dimx + 1;
@@ -187,7 +190,7 @@ __device__ void run_syevj(const rocblas_int dimx,
                     aij = Acpy[i + j * n];
                     mag = std::abs(aij);
 
-                    if(mag < eps)
+                    if(mag * mag < small_num)
                     {
                         c = 1;
                         s1 = 0;
@@ -390,8 +393,8 @@ __host__ __device__ inline void
 
     rocblas_int even_n = n + n % 2;
     rocblas_int half_n = even_n / 2;
-    rocblas_int y = min(bdim / 4, half_n);
-    rocblas_int x = min(bdim / y, half_n);
+    rocblas_int y = std::min(bdim / 4, half_n);
+    rocblas_int x = std::min(bdim / y, half_n);
     *ddx = x;
     *ddy = y;
 }
@@ -474,6 +477,7 @@ ROCSOLVER_KERNEL void syevj_init(const rocblas_evect evect,
 {
     rocblas_int tid = hipThreadIdx_x;
     rocblas_int bid = hipBlockIdx_y;
+    rocblas_int dimx = hipBlockDim_x;
 
     // local variables
     T temp;
@@ -488,7 +492,7 @@ ROCSOLVER_KERNEL void syevj_init(const rocblas_evect evect,
     // shared memory
     extern __shared__ double lmem[];
     S* sh_res = reinterpret_cast<S*>(lmem);
-    S* sh_diag = sh_res + hipBlockDim_x;
+    S* sh_diag = sh_res + dimx;
 
     // copy A to Acpy, set A to identity (if calculating eigenvectors), and calculate off-diagonal
     // squared Frobenius norm (by column/row)
@@ -496,7 +500,7 @@ ROCSOLVER_KERNEL void syevj_init(const rocblas_evect evect,
     S local_diag = 0;
     if(uplo == rocblas_fill_upper)
     {
-        for(i = tid; i < n; i += hipBlockDim_x)
+        for(i = tid; i < n; i += dimx)
         {
             temp = A[i + i * lda];
             local_diag += std::norm(temp);
@@ -522,7 +526,7 @@ ROCSOLVER_KERNEL void syevj_init(const rocblas_evect evect,
     }
     else
     {
-        for(i = tid; i < n; i += hipBlockDim_x)
+        for(i = tid; i < n; i += dimx)
         {
             temp = A[i + i * lda];
             local_diag += std::norm(temp);
@@ -552,7 +556,7 @@ ROCSOLVER_KERNEL void syevj_init(const rocblas_evect evect,
 
     if(tid == 0)
     {
-        for(i = 1; i < min(n, hipBlockDim_x); i++)
+        for(i = 1; i < std::min(n, dimx); i++)
         {
             local_res += sh_res[i];
             local_diag += sh_diag[i];
@@ -570,7 +574,7 @@ ROCSOLVER_KERNEL void syevj_init(const rocblas_evect evect,
     // initialize top/bottom pairs
     if(bid == 0 && top && bottom)
     {
-        for(i = tid; i < half_blocks; i += hipBlockDim_x)
+        for(i = tid; i < half_blocks; i += dimx)
         {
             top[i] = 2 * i;
             bottom[i] = 2 * i + 1;
@@ -618,7 +622,7 @@ ROCSOLVER_KERNEL void syevj_diag_kernel(const rocblas_int n,
     rocblas_int y1 = yy1 + offset, y2 = y1 + 1;
 
     rocblas_int half_n = (n - 1) / 2 + 1;
-    rocblas_int nb = min(2 * half_n - offset, nb_max);
+    rocblas_int nb = std::min(2 * half_n - offset, nb_max);
     rocblas_int half_nb = nb / 2;
 
     if(tix >= half_nb || tiy >= half_nb)
@@ -651,6 +655,8 @@ ROCSOLVER_KERNEL void syevj_diag_kernel(const rocblas_int n,
         sh_bottom[tix] = x2;
     }
 
+    S small_num = get_safemin<S>() / eps;
+
     // for each off-diagonal element (indexed using top/bottom pairs), calculate the Jacobi rotation and apply it to A
     i = x1;
     j = x2;
@@ -662,7 +668,7 @@ ROCSOLVER_KERNEL void syevj_diag_kernel(const rocblas_int n,
             mag = std::abs(aij);
 
             // calculate rotation J
-            if(mag < eps)
+            if(mag * mag < small_num)
             {
                 c = 1;
                 s1 = 0;
@@ -804,7 +810,7 @@ ROCSOLVER_KERNEL void syevj_diag_rotate(const bool skip_block,
     rocblas_int x = tix + offsetx;
     rocblas_int y = tiy + offsety;
 
-    rocblas_int nb = min(n - offsetx, nb_max);
+    rocblas_int nb = std::min(n - offsetx, nb_max);
 
     if(x >= n || y >= n)
         return;
@@ -908,6 +914,8 @@ ROCSOLVER_KERNEL void syevj_offd_kernel(const rocblas_int blocks,
         J[xx2 + yy2 * ldj] = (xx2 == yy2 ? 1 : 0);
     }
 
+    S small_num = get_safemin<S>() / eps;
+
     // for each element, calculate the Jacobi rotation and apply it to A
     for(k = 0; k < nb_max; k++)
     {
@@ -921,7 +929,7 @@ ROCSOLVER_KERNEL void syevj_offd_kernel(const rocblas_int blocks,
             mag = std::abs(aij);
 
             // calculate rotation J
-            if(mag < eps)
+            if(mag * mag < small_num)
             {
                 c = 1;
                 s1 = 0;
@@ -1057,7 +1065,7 @@ ROCSOLVER_KERNEL void syevj_offd_rotate(const bool skip_block,
     rocblas_int x = tix + offsetx;
     rocblas_int y = tiy + offsety;
 
-    rocblas_int nb = min(n - offsetj, nb_max);
+    rocblas_int nb = std::min(n - offsetj, nb_max);
 
     if(x >= n || y >= n)
         return;
@@ -1097,53 +1105,45 @@ template <typename T>
 ROCSOLVER_KERNEL void
     syevj_cycle_pairs(const rocblas_int half_blocks, rocblas_int* top, rocblas_int* bottom)
 {
-    rocblas_int tix = hipThreadIdx_x;
-    rocblas_int i, j, k;
+    rocblas_int n = half_blocks - 1;
 
-    if(half_blocks <= hipBlockDim_x && tix < half_blocks)
+    auto cycle = [n = n](auto i) -> auto
     {
-        if(tix == 0)
-            i = 0;
-        else if(tix == 1)
-            i = bottom[0];
-        else if(tix > 1)
-            i = top[tix - 1];
+        using I = decltype(i);
+        i = (i - 1) % (2 * n + 1) + 1;
+        I j{};
 
-        if(tix == half_blocks - 1)
-            j = top[half_blocks - 1];
+        if(i % 2 == 0)
+        {
+            j = i + 2;
+            if(j > 2 * n)
+            {
+                j = 2 * n + 1;
+            }
+        }
         else
-            j = bottom[tix + 1];
-        __syncthreads();
+        {
+            j = i - 2;
+            if(j < 1)
+            {
+                j = 2;
+            }
+        }
 
-        top[tix] = i;
-        bottom[tix] = j;
-    }
-    else
+        return j;
+    };
+
+    rocblas_int tidx = hipThreadIdx_x;
+    rocblas_int dimx = hipBlockDim_x;
+
+    if(tidx == 0)
     {
-        // shared memory
-        extern __shared__ double lmem[];
-        rocblas_int* sh_top = reinterpret_cast<rocblas_int*>(lmem);
-        rocblas_int* sh_bottom = reinterpret_cast<rocblas_int*>(sh_top + half_blocks);
-
-        for(k = tix; k < half_blocks; k += hipBlockDim_x)
-        {
-            sh_top[k] = top[k];
-            sh_bottom[k] = bottom[k];
-        }
-        __syncthreads();
-
-        for(k = tix; k < half_blocks; k += hipBlockDim_x)
-        {
-            if(k == 1)
-                top[k] = sh_bottom[0];
-            else if(k > 1)
-                top[k] = sh_top[k - 1];
-
-            if(k == half_blocks - 1)
-                bottom[k] = sh_top[half_blocks - 1];
-            else
-                bottom[k] = sh_bottom[k + 1];
-        }
+        bottom[0] = cycle(bottom[0]);
+    }
+    for(rocblas_int l = tidx + 1; l < half_blocks; l += dimx)
+    {
+        top[l] = cycle(top[l]);
+        bottom[l] = cycle(bottom[l]);
     }
 }
 
@@ -1160,6 +1160,7 @@ ROCSOLVER_KERNEL void syevj_calc_norm(const rocblas_int n,
 {
     rocblas_int tid = hipThreadIdx_x;
     rocblas_int bid = hipBlockIdx_y;
+    rocblas_int dimx = hipBlockDim_x;
 
     if(completed[bid + 1])
         return;
@@ -1175,7 +1176,7 @@ ROCSOLVER_KERNEL void syevj_calc_norm(const rocblas_int n,
     S* sh_res = reinterpret_cast<S*>(lmem);
 
     S local_res = 0;
-    for(i = tid; i < n; i += hipBlockDim_x)
+    for(i = tid; i < n; i += dimx)
     {
         for(j = 0; j < i; j++)
             local_res += 2 * std::norm(Acpy[i + j * n]);
@@ -1185,7 +1186,7 @@ ROCSOLVER_KERNEL void syevj_calc_norm(const rocblas_int n,
 
     if(tid == 0)
     {
-        for(i = 1; i < min(n, hipBlockDim_x); i++)
+        for(i = 1; i < std::min(n, dimx); i++)
             local_res += sh_res[i];
 
         residual[bid] = local_res;
@@ -1599,3 +1600,5 @@ rocblas_status rocsolver_syevj_heevj_template(rocblas_handle handle,
 
     return rocblas_status_success;
 }
+
+ROCSOLVER_END_NAMESPACE
